import json
import requests
from flask import Response
from xml.etree import ElementTree as ET
import xmlschema
import sys
from urllib.parse import urlparse
#from pprint import pprint

# https://stackoverflow.com/questions/7160737/python-how-to-validate-a-url-in-python-malformed-or-not
def uri_validator(x):
    try:
        result = urlparse(x)
        return all([result.scheme, result.netloc, result.path])
    except:
        return False

def get_config_dict():
    with open('./config.json') as json_config_file:
        config = json.load(json_config_file)

    return config

def get_version():
    try:
        with open('./VERSION', 'r') as f:
            version = f.read().rstrip()
        return version
    except:
        return False

def get_xsd(cfg, xsd_version=None, xml_str = None):
    filename =  cfg['xsd']['filename']
    baseURL = cfg['xsd']['externalBaseURL']
    xsd_str = ''
    url = ''
    if xsd_version:
        if xsd_version == 'latest':
            url = baseURL + filename
        else:
            url = baseURL + 'v' + xsd_version + '/' + filename
    else:
        root = parse(xml_str)
        item = root.attrib.get('schemaVersion')
        if item:
            url = baseURL + 'v' + item + '/' + filename
        else:
            for key in root.attrib:
                if key.endswith('schemaLocation'):
                    x = root.attrib.get(key).split()
                    for u in x:
                        if u.endswith(filename):
                            url = u
                            break       
    if uri_validator(url):
        try:
            r = requests.get(url, allow_redirects=True)
            if r.url.endswith(filename): # no bad redirection
                return r.text
            else:
                return False
        except:
            return False

def parse(xml_str):
    try:
        tree = ET.fromstring(xml_str)
    except ET.ParseError:
        tree = None
    return tree

def validate(xml_str, xsd_str):
    try:
        tree = parse(xml_str)
        schema = xmlschema.XMLSchema(xsd_str)
        schema.validate(tree)
        return return_ok() 
    except Exception as error:
        return return_error(error)
   
def xml_response(xml_str):
    return Response(xml_str, content_type='text/xml; charset=utf-8')

def return_ok():
    return '<ok/>\n'

def return_error(error):
    return '<error>{error}</error>\n'.format(error=error)

def return_version(version):
    return '<version>{version}</version>\n'.format(version=version)
